@@ -56,9 +56,10 @@ def about():
     print("================================")
     print("Copyright (c) QuTiP team 2011 and later.")
     print("Original developers: R. J. Johansson & P. D. Nation.")
+    print("Previous lead developers: Chris Granade & A. Grimsmo.")
     print("Current admin team: Alexander Pitchford, Paul D. Nation, "
             "Nathan Shammah, Shahnawaz Ahmed, "
-            "Neill Lambert, and Eric Giguère.")
+            "Neill Lambert, Eric Giguère, and Boxi Li")
     print("Project Manager: Franco Nori.")
     print("Currently developed through wide collaboration. "
           "See https://github.com/qutip for details.")
