@@ -37,10 +37,8 @@ import inspect
 
 import numpy as np
 
-import numpy as np
-
 from qutip.qip.circuit_latex import _latex_compile
-from qutip.qip.gates import *
+from qutip.qip.operations.gates import *
 from qutip.qip.qubits import qubit_states
 
 __all__ = ['Gate', 'QubitCircuit']
@@ -181,6 +179,10 @@ class QubitCircuit(object):
         Define a dictionary of the custom gates. See examples for detail.
     input_states : list
         A list of string such as `0`,'+', "A", "Y". Only used for latex.
+    dims : list
+        A list of integer for the dimension of each composite system.
+        e.g [2,2,2,2,2] for 5 qubits system. If None, qubits system
+        will be the default option.
 
     Examples
     --------
@@ -193,7 +195,7 @@ class QubitCircuit(object):
     """
 
     def __init__(self, N, input_states=None, output_states=None,
-                 reverse_states=True, user_gates=None):
+                 reverse_states=True, user_gates=None, dims=None):
         # number of qubits in the register
         self.N = N
         self.reverse_states = reverse_states
@@ -201,6 +203,7 @@ class QubitCircuit(object):
         self.U_list = []
         self.input_states = [None for i in range(N)]
         self.output_states = [None for i in range(N)]
+        self.dims = dims
         if user_gates is None:
             self.user_gates = {}
         else:
@@ -409,7 +412,7 @@ class QubitCircuit(object):
         Reverse an entire circuit of unitary gates.
 
         Returns
-        ----------
+        -------
         qc : QubitCircuit
             Return QubitCircuit of resolved gates for the qubit circuit in the
             reverse order.
@@ -487,11 +490,11 @@ class QubitCircuit(object):
                 temp_resolved.append(Gate("GLOBALPHASE", None, None,
                                           arg_value=np.pi / 2,
                                           arg_label=r"\pi/2"))
-                temp_resolved.append(Gate("RX", gate.targets, None,
-                                          arg_value=np.pi, arg_label=r"\pi"))
                 temp_resolved.append(Gate("RY", gate.targets, None,
                                           arg_value=np.pi / 2,
                                           arg_label=r"\pi/2"))
+                temp_resolved.append(Gate("RX", gate.targets, None,
+                                          arg_value=np.pi, arg_label=r"\pi"))
             elif gate.name == "PHASEGATE":
                 temp_resolved.append(Gate("GLOBALPHASE", None, None,
                                           arg_value=gate.arg_value / 2,
@@ -1028,7 +1031,9 @@ class QubitCircuit(object):
                 else:
                     raise ValueError(
                         "gate function takes at most one parameters.")
-                self.U_list.append(expand_oper(oper, self.N, gate.targets))
+                self.U_list.append(expand_operator(
+                        oper, N=self.N,
+                        targets=gate.targets, dims=self.dims))
 
             else:
                 raise NotImplementedError(
