@@ -33,10 +33,14 @@
 
 import numpy as np
 from numpy.testing import assert_, assert_allclose, run_module_suite
-from qutip.qip.gates import gate_sequence_product, rx, identity, expand_oper
+from qutip.qip.operations.gates import (
+    gate_sequence_product, rx, expand_operator)
+from qutip.operators import identity
 from qutip.qip.circuit import QubitCircuit, Gate
 from qutip.tensor import tensor
-from qutip.qobj import Qobj
+from qutip.qobj import Qobj, ptrace
+from qutip.random_objects import rand_dm
+from qutip.states import fock_dm
 
 
 class TestQubitCircuit:
@@ -128,6 +132,18 @@ class TestQubitCircuit:
         U2 = gate_sequence_product(qc2.propagators())
         assert_((U1 - U2).norm() < 1e-12)
 
+    def testSNOTdecompose(self):
+        """
+        SNOT to rotation: compare unitary matrix for SNOT and product of
+        resolved matrices in terms of rotation gates.
+        """
+        qc1 = QubitCircuit(1)
+        qc1.add_gate("SNOT", targets=0)
+        U1 = gate_sequence_product(qc1.propagators())
+        qc2 = qc1.resolve_gates()
+        U2 = gate_sequence_product(qc2.propagators())
+        assert_((U1 - U2).norm() < 1e-12)
+
     def testadjacentgates(self):
         """
         Adjacent Gates: compare unitary matrix for ISWAP and product of
@@ -257,6 +273,27 @@ class TestQubitCircuit:
         result2 = tensor(identity(2), customer_gate2(None), identity(2))
         assert_allclose(props[1], result2)
 
+    def test_N_level_system(self):
+        """
+        Test for circuit with N-level system.
+        """
+        mat3 = rand_dm(3, density=1.)
+
+        def controlled_mat3(arg_value):
+            """
+            A qubit control an operator acting on a 3 level system
+            """
+            control_value = arg_value
+            dim = mat3.dims[0][0]
+            return (tensor(fock_dm(2, control_value), mat3) +
+                    tensor(fock_dm(2, 1 - control_value), identity(dim)))        
+
+        qc = QubitCircuit(2, dims=[3, 2])
+        qc.user_gates = {"CTRLMAT3": controlled_mat3}
+        qc.add_gate("CTRLMAT3", targets=[1, 0], arg_value=1)
+        props = qc.propagators()
+        assert_allclose(mat3, ptrace(props[0], 0) - 1)
+
 
 if __name__ == "__main__":
     run_module_suite()
